/*
  crc32.cpp - CRC32 Library

  Copyright (c) 2022 Sasapea's Lab. All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/
#pragma once

#include <stdint.h>

class CRC32
{
  public:

    typedef enum
    {
      POLY_IEEE802_3 = 0xEDB88320, // CRC-32-IEEE 802.3
    } POLY;

    static
    void init(uint32_t initval = (uint32_t)-1, POLY poly = POLY_IEEE802_3)
    {
      _result = initval;
      for (uint32_t i = 0; i < 256; i++)
      {
        uint32_t c = i;
        for (int j = 0; j < 8; j++)
          c = c & 1 ? poly ^ (c >> 1) : c >> 1;
        _lookup[i] = c;
      }
    }

    static
    void update(const uint8_t *buf, uint32_t len)
    {
      for (uint32_t i = 0; i < len; i++)
        _result = _lookup[(_result ^ *buf++) & 0xFF] ^ (_result >> 8);
    }

    static
    uint32_t result(uint32_t xorout = (uint32_t)-1)
    {
      return _result ^ xorout;
    }

  private:

    static uint32_t _result;
    static uint32_t _lookup[256];
};
