/*
 ftd2.cpp - Serial Library for FTDI USB Chip FT2XX

 Copyright (c) 2022 Sasapea's Lab. All right reserved.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General
 Public License along with this library; if not, write to the
 Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 Boston, MA  02111-1307  USA
 */
#include "ftd2.h"

typedef FT_STATUS (WINAPI *LPFN_FT_ListDevices)(PVOID pvArg1, PVOID pvArg2, DWORD dwFlags);
typedef FT_STATUS (WINAPI *LPFN_FT_Open)(int iDevice, FT_HANDLE *ftHandle);
typedef FT_STATUS (WINAPI *LPFN_FT_OpenEx)(PVOID pArg1, DWORD Flags, FT_HANDLE *pHandle);
typedef FT_STATUS (WINAPI *LPFN_FT_Close)(FT_HANDLE ftHandle);
typedef FT_STATUS (WINAPI *LPFN_FT_SetTimeouts)(FT_HANDLE ftHandle, DWORD dwReadTimeout, DWORD dwWriteTimeout);
typedef FT_STATUS (WINAPI *LPFN_FT_Read)(FT_HANDLE ftHandle, LPVOID lpBuffer, DWORD dwBytesToRead, LPDWORD lpBytesReturned);
typedef FT_STATUS (WINAPI *LPFN_FT_Write)(FT_HANDLE ftHandle, LPVOID lpBuffer, DWORD dwBytesToWrite, LPDWORD lpBytesWritten);
typedef FT_STATUS (WINAPI *LPFN_FT_SetBaudRate)(FT_HANDLE ftHandle, ULONG BaudRate);
typedef FT_STATUS (WINAPI *LPFN_FT_SetDivisor)(FT_HANDLE ftHandle, USHORT usDivisor);
typedef FT_STATUS (WINAPI *LPFN_FT_SetDataCharacteristics)(FT_HANDLE ftHandle, UCHAR WordLength, UCHAR StopBits, UCHAR Parity);
typedef FT_STATUS (WINAPI *LPFN_FT_SetFlowControl)(FT_HANDLE ftHandle, USHORT FlowControl, UCHAR XonChar, UCHAR XoffChar);
typedef FT_STATUS (WINAPI *LPFN_FT_SetDtr)(FT_HANDLE ftHandle);
typedef FT_STATUS (WINAPI *LPFN_FT_ClrDtr)(FT_HANDLE ftHandle);
typedef FT_STATUS (WINAPI *LPFN_FT_SetRts)(FT_HANDLE ftHandle);
typedef FT_STATUS (WINAPI *LPFN_FT_ClrRts)(FT_HANDLE ftHandle);
typedef FT_STATUS (WINAPI *LPFN_FT_SetBitMode)(FT_HANDLE ftHandle, UCHAR ucMask, UCHAR ucEnable);
typedef FT_STATUS (WINAPI *LPFN_FT_GetBitMode)(FT_HANDLE ftHandle, PUCHAR pucMode);
typedef FT_STATUS (WINAPI *LPFN_FT_GetComPortNumber)(FT_HANDLE ftHandle, LPLONG lplComPortNumber);
typedef FT_STATUS (WINAPI *LPFN_FT_Purge)(FT_HANDLE ftHandle, ULONG Mask);

static HMODULE hFTD2XX;
static LPFN_FT_ListDevices lpfn_FT_ListDevices;
static LPFN_FT_Open lpfn_FT_Open;
static LPFN_FT_OpenEx lpfn_FT_OpenEx;
static LPFN_FT_Close lpfn_FT_Close;
static LPFN_FT_SetTimeouts lpfn_FT_SetTimeouts;
static LPFN_FT_Read lpfn_FT_Read;
static LPFN_FT_Write lpfn_FT_Write;
static LPFN_FT_SetBaudRate lpfn_FT_SetBaudRate;
static LPFN_FT_SetDivisor lpfn_FT_SetDivisor;
static LPFN_FT_SetDataCharacteristics lpfn_FT_SetDataCharacteristics;
static LPFN_FT_SetFlowControl lpfn_FT_SetFlowControl;
static LPFN_FT_SetDtr lpfn_FT_SetDtr;
static LPFN_FT_ClrDtr lpfn_FT_ClrDtr;
static LPFN_FT_SetRts lpfn_FT_SetRts;
static LPFN_FT_ClrRts lpfn_FT_ClrRts;
static LPFN_FT_SetBitMode lpfn_FT_SetBitMode;
static LPFN_FT_GetBitMode lpfn_FT_GetBitMode;
static LPFN_FT_GetComPortNumber lpfn_FT_GetComPortNumber;
static LPFN_FT_Purge lpfn_FT_Purge;

static FT_STATUS status = FT_OK;

FT_STATUS FT_GetLastError(void)
{
  return status;
}

FT_STATUS WINAPI FT_Initialise(void)
{
  if (!hFTD2XX)
  {
    if (!(hFTD2XX = LoadLibrary("ftd2xx.dll")))
      return status = FT_NOT_SUPPORTED;
    lpfn_FT_ListDevices            = (LPFN_FT_ListDevices)           GetProcAddress(hFTD2XX, "FT_ListDevices");
    lpfn_FT_Open                   = (LPFN_FT_Open)                  GetProcAddress(hFTD2XX, "FT_Open");
    lpfn_FT_OpenEx                 = (LPFN_FT_OpenEx)                GetProcAddress(hFTD2XX, "FT_OpenEx");
    lpfn_FT_Close                  = (LPFN_FT_Close)                 GetProcAddress(hFTD2XX, "FT_Close");
    lpfn_FT_SetTimeouts            = (LPFN_FT_SetTimeouts)           GetProcAddress(hFTD2XX, "FT_SetTimeouts");
    lpfn_FT_Read                   = (LPFN_FT_Read)                  GetProcAddress(hFTD2XX, "FT_Read");
    lpfn_FT_Write                  = (LPFN_FT_Write)                 GetProcAddress(hFTD2XX, "FT_Write");
    lpfn_FT_SetBaudRate            = (LPFN_FT_SetBaudRate)           GetProcAddress(hFTD2XX, "FT_SetBaudRate");
    lpfn_FT_SetDivisor             = (LPFN_FT_SetDivisor)            GetProcAddress(hFTD2XX, "FT_SetDivisor");
    lpfn_FT_SetDataCharacteristics = (LPFN_FT_SetDataCharacteristics)GetProcAddress(hFTD2XX, "FT_SetDataCharacteristics");
    lpfn_FT_SetFlowControl         = (LPFN_FT_SetFlowControl)        GetProcAddress(hFTD2XX, "FT_SetFlowControl");
    lpfn_FT_SetDtr                 = (LPFN_FT_SetDtr)                GetProcAddress(hFTD2XX, "FT_SetDtr");
    lpfn_FT_ClrDtr                 = (LPFN_FT_ClrDtr)                GetProcAddress(hFTD2XX, "FT_ClrDtr");
    lpfn_FT_SetRts                 = (LPFN_FT_SetRts)                GetProcAddress(hFTD2XX, "FT_SetRts");
    lpfn_FT_ClrRts                 = (LPFN_FT_ClrRts)                GetProcAddress(hFTD2XX, "FT_ClrRts");
    lpfn_FT_SetBitMode             = (LPFN_FT_SetBitMode)            GetProcAddress(hFTD2XX, "FT_SetBitMode");
    lpfn_FT_GetBitMode             = (LPFN_FT_GetBitMode)            GetProcAddress(hFTD2XX, "FT_GetBitMode");
    lpfn_FT_GetComPortNumber       = (LPFN_FT_GetComPortNumber)      GetProcAddress(hFTD2XX, "FT_GetComPortNumber");
    lpfn_FT_Purge                  = (LPFN_FT_Purge)                 GetProcAddress(hFTD2XX, "FN_FT_Purge");
  }
  return status = FT_OK;
}

void WINAPI FT_Finalise(void)
{
  if (hFTD2XX)
  {
    FreeLibrary(hFTD2XX);
    hFTD2XX = NULL;
  }
}

FTD2XX_API FT_STATUS FT_ListDevices(PVOID pvArg1, PVOID pvArg2, DWORD dwFlags)
{
  return status = (lpfn_FT_ListDevices ? lpfn_FT_ListDevices(pvArg1, pvArg2, dwFlags) : FT_NOT_SUPPORTED);
}

FTD2XX_API FT_STATUS FT_Open(int iDevice, FT_HANDLE *ftHandle)
{
  return status = (lpfn_FT_Open ? lpfn_FT_Open(iDevice, ftHandle) : FT_NOT_SUPPORTED);
}

FTD2XX_API FT_STATUS FT_OpenEx(PVOID pArg1, DWORD Flags, FT_HANDLE *pHandle)
{
  return status = (lpfn_FT_OpenEx ? lpfn_FT_OpenEx(pArg1, Flags, pHandle) : FT_NOT_SUPPORTED);
}

FTD2XX_API FT_STATUS FT_Close(FT_HANDLE ftHandle)
{
  return status = (lpfn_FT_Close ? lpfn_FT_Close(ftHandle) : FT_NOT_SUPPORTED);
}

FTD2XX_API FT_STATUS FT_SetTimeouts(FT_HANDLE ftHandle, DWORD dwReadTimeout, DWORD dwWriteTimeout)
{
  return status = (lpfn_FT_SetTimeouts ? lpfn_FT_SetTimeouts(ftHandle, dwReadTimeout, dwWriteTimeout) : FT_NOT_SUPPORTED);
}

FTD2XX_API FT_STATUS FT_Read(FT_HANDLE ftHandle, LPVOID lpBuffer, DWORD dwBytesToRead, LPDWORD lpBytesReturned)
{
  return status = (lpfn_FT_Read ? lpfn_FT_Read(ftHandle, lpBuffer, dwBytesToRead, lpBytesReturned) : FT_NOT_SUPPORTED);
}

FTD2XX_API FT_STATUS FT_Write(FT_HANDLE ftHandle, LPVOID lpBuffer, DWORD dwBytesToWrite, LPDWORD lpBytesWritten)
{
  return status = (lpfn_FT_Write ? lpfn_FT_Write(ftHandle, lpBuffer, dwBytesToWrite, lpBytesWritten) : FT_NOT_SUPPORTED);
}

FTD2XX_API FT_STATUS FT_SetBaudRate(FT_HANDLE ftHandle, ULONG BaudRate)
{
  return status = (lpfn_FT_SetBaudRate ? lpfn_FT_SetBaudRate(ftHandle, BaudRate) : FT_NOT_SUPPORTED);
}

FTD2XX_API FT_STATUS FT_SetDivisor(FT_HANDLE ftHandle, USHORT usDivisor)
{
  return status = (lpfn_FT_SetDivisor ? lpfn_FT_SetDivisor(ftHandle, usDivisor) : FT_NOT_SUPPORTED);
}

FTD2XX_API FT_STATUS FT_SetDataCharacteristics(FT_HANDLE ftHandle, UCHAR WordLength, UCHAR StopBits, UCHAR Parity)
{
  return status = (lpfn_FT_SetDataCharacteristics ? lpfn_FT_SetDataCharacteristics(ftHandle, WordLength, StopBits, Parity) : FT_NOT_SUPPORTED);
}

FTD2XX_API FT_STATUS FT_SetFlowControl(FT_HANDLE ftHandle, USHORT FlowControl, UCHAR XonChar, UCHAR XoffChar)
{
  return status = (lpfn_FT_SetFlowControl ? lpfn_FT_SetFlowControl(ftHandle, FlowControl, XonChar, XoffChar) : FT_NOT_SUPPORTED);
}

FTD2XX_API FT_STATUS FT_SetDtr(FT_HANDLE ftHandle)
{
  return status = (lpfn_FT_SetDtr ? lpfn_FT_SetDtr(ftHandle) : FT_NOT_SUPPORTED);
}

FTD2XX_API FT_STATUS FT_ClrDtr(FT_HANDLE ftHandle)
{
  return status = (lpfn_FT_ClrDtr ? lpfn_FT_ClrDtr(ftHandle) : FT_NOT_SUPPORTED);
}

FTD2XX_API FT_STATUS FT_SetRts(FT_HANDLE ftHandle)
{
  return status = (lpfn_FT_SetRts ? lpfn_FT_SetRts(ftHandle) : FT_NOT_SUPPORTED);
}

FTD2XX_API FT_STATUS FT_ClrRts(FT_HANDLE ftHandle)
{
  return status = (lpfn_FT_ClrRts ? lpfn_FT_ClrRts(ftHandle) : FT_NOT_SUPPORTED);
}

FTD2XX_API FT_STATUS FT_SetBitMode(FT_HANDLE ftHandle, UCHAR ucMask, UCHAR ucEnable)
{
  return status = (lpfn_FT_SetBitMode ? lpfn_FT_SetBitMode(ftHandle, ucMask, ucEnable) : FT_NOT_SUPPORTED);
}

FTD2XX_API FT_STATUS FT_GetBitMode(FT_HANDLE ftHandle, PUCHAR pucMode)
{
  return status = (lpfn_FT_GetBitMode ? lpfn_FT_GetBitMode(ftHandle, pucMode) : FT_NOT_SUPPORTED);
}

FTD2XX_API FT_STATUS FT_GetComPortNumber(FT_HANDLE ftHandle, LPLONG lplComPortNumber)
{
  return status = (lpfn_FT_GetComPortNumber ? lpfn_FT_GetComPortNumber(ftHandle, lplComPortNumber) : FT_NOT_SUPPORTED);
}

FTD2XX_API FT_STATUS FT_Purge(FT_HANDLE ftHandle, ULONG Mask)
{
  return status = (lpfn_FT_Purge ? lpfn_FT_Purge(ftHandle, Mask) : FT_NOT_SUPPORTED);
}
